""" Game fix for FINAL FANTASY IX
"""
#pylint: disable=C0103


import os
import sys


def main():
    """ Changes the proton argument from the launcher to the game
    """

    print('Applying FINAL FANTASY IX Game Fixes')

    # Fix crackling audio
    os.environ['PULSE_LATENCY_MSEC'] = '60'

    # Replace launcher with game exe in proton arguments
    for idx, env in enumerate(sys.argv):
        if 'FF9_Launcher' in env:
            sys.argv[idx] = env.replace('FF9_Launcher.exe', 'x64/FF9.exe')



""" Splash screen for protonfixes using cefpython
"""
import sys
from os import path
from time import sleep
from multiprocessing import Process
from contextlib import contextmanager
from cefpython3 import cefpython as cef

# pylint: disable=I1101

def browser(url):
    """ Starts a cef browser in the middle of the screen with url
    """

    # Keeps the splash from displaying on short tasks
    sleep(2)

    sys.excepthook = cef.ExceptHook
    settings = {
        'background_color': 0xff000000,
        'cache_path': '',
        'context_menu': {'enabled': False},
        'debug': False,
        'command_line_args_disabled': False,
    }
    switches = {
        'disable-gpu': '',
        'disable-gpu-compositing': '',
    }

    cef.Initialize(settings, switches)

    win_info = cef.WindowInfo()
    win_info.SetAsChild(0, coordinates(600, 360))

    cef.CreateBrowser(url=url, window_info=win_info, window_title='splash')
    cef.MessageLoop()
    cef.Shutdown()

def coordinates(width, height):
    """ Returns coordinates [x1, y1, x2, y2] for a centered box of width, height
    """

    with open('/sys/class/graphics/fb0/virtual_size', 'r') as res:
        screenx, screeny = map(int, res.read().strip('\n').split(','))

    posx = (screenx/2) - (width/2)
    posy = (screeny/2) - (height/2)

    return [posx, posy, posx+width, posy+height]

@contextmanager
def splash(page='index.html'):
    """ Runs the browser in a seperate process until the context is returned
    """

    data_dir = path.join(path.dirname(__file__), '..', 'static')
    url = 'file://' + path.join(data_dir, page)
    splashwin = Process(target=browser, args=(url,))
    splashwin.start()
    yield
    splashwin.terminate()

def _test():
    """ Used for testing the splash with python -m splash
    """

    import time
    with splash():
        while True:
            time.sleep(1)

if __name__ == '__main__':
    _test()



""" Utilities to make gamefixes easier
"""

import os
import sys
import glob
import shutil
import signal
import subprocess

# pylint: disable=I1101, W0101

def which(appname):
    """ Returns the full path of an executable in $PATH
    """

    for path in os.environ['PATH'].split(os.pathsep):
        fullpath = os.path.join(path, appname)
        if os.path.exists(fullpath) and os.access(fullpath, os.X_OK):
            return fullpath
    print('%s not found in $PATH')
    return None


def protondir():
    """ Returns the path to proton
    """
    
    proton_dir = os.path.dirname(sys.argv[0])
    return proton_dir


def protonprefix():
    """ Returns the wineprefix used by proton
    """

    return os.path.join(
        os.environ['STEAM_COMPAT_DATA_PATH'],
        'pfx/')


def _killhanging():
    """ Kills processes that hang when installing winetricks
    """

    # avoiding an external library as proc should be available on linux
    pids = [pid for pid in os.listdir('/proc') if pid.isdigit()]
    badexes = ['mscorsvw.exe']
    for pid in pids:
        try:
            with open(os.path.join('/proc', pid, 'cmdline'), 'rb') as proc_cmd:
                cmdline = proc_cmd.read()
                for exe in badexes:
                    if exe in cmdline.decode():
                        os.kill(int(pid), signal.SIGKILL)
        except IOError:
            continue


def protontricks(verb):
    """ Runs winetricks if available
    """

    if not checkinstalled(verb):
        env = dict(os.environ)
        env['WINEPREFIX'] = protonprefix()
        env['WINESERVER'] = os.path.join(protondir(), 'dist/bin/wineserver')

        winetricks_bin = which('winetricks')
        winetricks_cmd = [winetricks_bin, '--unattended', '--force'] + verb.split(' ')
        wineserver_bin = env['WINESERVER']

        if winetricks_bin is not None:

            # winetricks relies entirely on the existence of syswow64 to determine
            # if the prefix is 64 bit, while proton fails to run without it
            if 'win32' in protonprefix():
                try:
                    shutil.rmtree(os.path.join(protonprefix(), 'drive_c/windows/syswow64'))
                except FileNotFoundError:
                    pass

            # make sure proton waits for winetricks to finish
            for idx, arg in enumerate(sys.argv):
                if 'waitforexitandrun' not in arg:
                    sys.argv[idx] = arg.replace('run', 'waitforexitandrun')

            print('Using winetricks', verb)
            process = subprocess.Popen(winetricks_cmd, env=env)
            process.wait()
            _killhanging()
            subprocess.Popen([wineserver_bin, '-k'], env=env)
            return True

            # restore syswow64 so proton doesn't crash
            if 'win32' in protonprefix():
                try:
                    os.makedirs(os.path.join(protonprefix(), 'drive_c/windows/syswow64'))
                except FileExistsError:
                    pass

    return False


def checkinstalled(verb):
    """ Returns True if the winetricks verb is found in the winetricks log
    """

    winetricks_log = os.path.join(protonprefix(), 'winetricks.log')
    try:
        with open(winetricks_log, 'r') as log:
            if verb in [x.strip() for x in log.readlines()]:
                return True
    except OSError:
        return False
    return False


def win32_prefix_exists():
    """ Returns True if there is a _win32 path available
    """

    prefix32 = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx'
    if os.path.exists(prefix32):
        return True
    return False


def use_win32_prefix():
    """ Sets variables in the main proton script to use the _win32 prefix
    """
    if not win32_prefix_exists():
        make_win32_prefix()

    import __main__ as protonmain
    data_path = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32'
    prefix32 = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx/'

    protonmain.env['STEAM_COMPAT_DATA_PATH'] = data_path
    protonmain.env['WINEPREFIX'] = prefix32
    protonmain.env['WINEDLLPATH'] = os.path.join(protondir(), 'dist/lib/wine')
    protonmain.env['WINEARCH'] = 'win32'
    protonmain.prefix = prefix32

    os.environ['STEAM_COMPAT_DATA_PATH'] = data_path
    os.environ['WINEPREFIX'] = prefix32
    os.environ['WINEDLLPATH'] = os.path.join(protondir(), 'dist/lib/wine')
    os.environ['WINEARCH'] = 'win32'

    # make sure steam doesn't crash when missing syswow
    try:
        os.makedirs(os.path.join(prefix32, 'drive_c/windows/syswow64'))
    except FileExistsError:
        pass


def make_win32_prefix():
    """ Creates a win32 prefix
    """

    env = dict(os.environ)
    print('Bootstrapping win32 prefix')

    prefix32 = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx'
    try:
        os.makedirs(prefix32)

        env['WINEARCH'] = 'win32'
        env['WINEPREFIX'] = prefix32
        server = subprocess.Popen([which('wineserver'), '-f'], env=env)
        subprocess.Popen([which('wine'), 'wineboot', '--init'], env=env)
        server.wait()

        os.makedirs(os.path.join(prefix32, 'drive_c/windows/syswow64'))
        os.symlink(
            os.path.join(prefix32, 'drive_c/Program Files'),
            os.path.join(prefix32, 'drive_c/Program Files (x86)'))

        print('Initialized win32 prefix')

    except OSError:
        print('Directory for win32 prefix already exists')


def replace_command(orig_str, repl_str):
    """ Make a commandline replacement in sys.argv
    """

    for idx, arg in enumerate(sys.argv):
        if orig_str in arg:
            sys.argv[idx] = arg.replace(orig_str, repl_str)

def append_argument(argument):
    """ Append an argument to sys.argv
    """

    sys.argv.append(argument)



""" Game fox for Oddworld: Abe's Oddysee
TODO: Fix steam controller input, it is stuck in lizard mode without overlay
"""
#pylint: disable=C0103


import sys


def main():
    """ Adds the -interline argument to the game
    """

    print('Applying Oddworld: Abe\'s Oddysee Fixes')

    # Adding -interline fixes slow videos but adds scanlines
    sys.argv.append('-interline')



""" Prints debug info if the environment variable DEBUG is 1
"""

import os
import sys
import shutil
# pylint: disable=E0611
from __main__ import CURRENT_PREFIX_VERSION, basedir, env

def log(msg=None):
    pfx = 'ProtonFixes[' + str(os.getpid()) + '] DEBUG: '
    sys.stderr.write(pfx + str(msg) +  os.linesep)
    sys.stderr.flush()

line = '---------------------------------------'
log('---- begin protontricks debug info ----')
log('Proton Python Version:')
log(sys.executable)
log(sys.version)
log(line)
log('System Python Version:')
try:
    log(shutil.which(os.readlink(shutil.which('python'))))
except:
    log(shutil.which('python'))
log(line)

log('Proton Version:')
log(CURRENT_PREFIX_VERSION)
log(line)

log('Proton Directory:')
log(basedir)
log(line)

ignorevars = ['SteamUser', 'OLDPWD', 'SDL_GAMECONTROLLERCONFIG', 'SteamAppUser', 'SDL_GAMECONTROLLER_IGNORE_DEVICES']
log('Environment Variables:')
for key, value in os.environ.items():
    if key not in ignorevars:
        log(key + '=' + value)
log(line)

log('Command Line:')
log(sys.argv)
log('----- end protontricks debug info -----')



""" Game fox for Oddworld: Munch's Oddysee
Work in progress
"""
#pylint: disable=C0103


import sys


def main():
    """ Changes the proton argument from the launcher to the game
    """

    print('Applying Oddworld: Munch\'s Oddysee Game Fixes')

    # Replace launcher with game exe in proton arguments
    for idx, env in enumerate(sys.argv):
        if 'Launcher' in env:
            sys.argv[idx] = env.replace('bin/Launcher.exe', 'Munch.exe')

    print(sys.argv)



""" Game fix for Styx: Master of Shadows
"""
#pylint: disable=C0103


from protonfixes import util


def main():
    """ Install dotnet40
    """

    print('Applying fixes for Styx: Master of Shadows')

    util.protontricks('dotnet40')
